#!/bin/bash

################################################################################
# (c) Copyright 2018 Xilinx, Inc. All rights reserved.
#
# This file contains confidential and proprietary information
# of Xilinx, Inc. and is protected under U.S. and
# international copyright and other intellectual property
# laws.
#
# DISCLAIMER
# This disclaimer is not a license and does not grant any
# rights to the materials distributed herewith. Except as
# otherwise provided in a valid license issued to you by
# Xilinx, and to the maximum extent permitted by applicable
# law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
# WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
# AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
# BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
# INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
# (2) Xilinx shall not be liable (whether in contract or tort,
# including negligence, or under any other theory of
# liability) for any loss or damage of any kind or nature
# related to, arising under or in connection with these
# materials, including for any direct, or any indirect,
# special, incidental, or consequential loss or damage
# (including loss of data, profits, goodwill, or any type of
# loss or damage suffered as a result of any action brought
# by a third party) even if such damage or loss was
# reasonably foreseeable or Xilinx had been advised of the
# possibility of the same.
#
# CRITICAL APPLICATIONS
# Xilinx products are not designed or intended to be fail-
# safe, or for use in any application requiring fail-safe
# performance, such as life-support or safety devices or
# systems, Class III medical devices, nuclear facilities,
# applications related to the deployment of airbags, or any
# other applications that could lead to death, personal
# injury, or severe property or environmental damage
# (individually and collectively, "Critical
# Applications"). Customer assumes the sole risk and
# liability of any use of Xilinx products in Critical
# Applications, subject only to applicable laws and
# regulations governing limitations on product liability.
#
# THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
# PART OF THIS FILE AT ALL TIMES.
################################################################################
# Description : Script use to setup a petalinux project based on an input BSP and
#               add SD-FEC Linux demo to the project
################################################################################

# Terminate build if a command fails
set -e

SCRIPTNAME=`basename "${0}"`


################################################################################
# Function implementations
################################################################################

################################################################################
# Setup Command
################################################################################

# to help setting up a work area
# and assoicated help
PrintSetupHelp()
{
    printf "Usage:\n"
    printf "\t./$SCRIPTNAME setup [options]\n"
    printf "Synopsis:\n"
    printf "\tSetups Petalinux Project \n"
    printf "Command:\n"
    printf "\t-h|--help : Print this help\n"
    printf "\t-b|--bsp  : Full path to BSP\n"
    printf "Example:\n"
    printf "\t./$SCRIPTNAME setup -d ./wrk \n"
    printf "\t\tsetup in the \"wrk\" directory\n"
}

SetupCmd()
{

    Dir="$PWD"

    while [[ $# -gt 0 ]]
    do
        Option="$1"
        shift
        case $Option in
            -h|--help)
                PrintSetupHelp
                exit 0
                ;;
            -b|--bsp)
                BspFullPath="$1"
                shift
                ;;
            *)
                echo "---- ERROR: Unknown option: $Option"
                PrintSetupHelp
                exit 1
            ;;
        esac
    done

    CurrentDir=$PWD
    cd $Dir

    echo "---- Setting up a Petalinux Project with BSP ${BspFullPath}"

	options="-bsp ${BspFullPath}"

    perl ./plnx_cmd_wrappers.pl generate $options

    cd $CurrentDir

}

################################################################################
# build command
################################################################################

# to help Building the project
PrintBuildHelp()
{
    printf "Usage:\n"
    printf "\t./$SCRIPTNAME build [options]\n"
    printf "Synopsis:\n"
    printf "\t builds the petalinux project \n"
    printf "Command:\n"
    printf "\t-h|--help           : Print this help\n"
    printf "\t-c <component>      : The component to be build as supported by petalinux tools \n"
    printf "\t-x <task>           : The task of the build process ( use do_listtasks to see what is possible, or omit) \n"
    printf "Example:\n"
    printf "\t./$SCRIPTNAME build -c sdfec-demo -x build\n"
    printf "\t\tbuild the sdfec-demo application and get get ready for download\n"
    printf "\n"
    printf "\t./$SCRIPTNAME build\n"
    printf "\t\t builds everything \n"
    printf "\n"
    printf "\t./$SCRIPTNAME build -c sdfec-demo -x clean\n"
    printf "\t\tclean the sdfec-demo application\n"
    printf "\n"
}


BuildCmd()
{
    # Default values
    Dir="$PWD"
    # Default to setup Both Git Repos and Petalinxu Project
    Component=""
    BuildTask=""

    while [[ $# -gt 0 ]]
    do
        Option="$1"
        shift
        case $Option in
            -h|--help)
                PrintBuildHelp
                exit 0
                ;;
            -c)
                Component="$1"
                shift
                ;;
            -x)
                BuildTask="$1"
                shift
                ;;
            *)
                echo "---- ERROR: Unknown option: $Option"
                PrintBuildHelp
                exit 1
            ;;
        esac
    done

    CurrentDir=$PWD
    cd $Dir

    Args=""
    if [[ $Component != "" ]]; then
        Args="$Args -c $Component";
    fi

    if [[ $BuildTask != "" ]]; then
        Args="$Args -x $BuildTask";
    fi

    echo "-*- $Args"
    perl ./plnx_cmd_wrappers.pl build $Args  |& tee build.log 
    cd $CurrentDir
}


################################################################################
# top level help
################################################################################
PrintHelp()
{
    printf "Usage:\n"
    printf "\t./$SCRIPTNAME [global options] <command> [options]\n"
    printf "Synopsis:\n"
    printf "\tAllow creation and building of a  SD FEC project\n"
    printf "Global Options:\n"
    printf "\t-h|--help:   print this help\n"
    printf "Command:\n"
    printf "\tsetup:   sets up working project areas\n"
    printf "\tbuild:   Build the software\n"
    printf "Example:\n"
    printf "\t./$SCRIPTNAME build -c kernel \n"
    printf "\t\tBuild the software\n"
    printf "\n"
    printf "\tfor additional help on comamnds use:\n"
    printf "\n"
    printf "\t$SCRIPTNAME <command> -h\n"
}

################################################################################
# Start of Execution
################################################################################


if [[ "${BASH_SOURCE[0]}" != "${0}" ]]
then
    echo "ERROR: Sourcing script is not supported - exiting"
else

    if [[ -z "$1" ]]
    then
        echo "ERROR: Insufficient arguments"
        PrintHelp
        exit 1
    fi

    # Extract mode selection from command line
    while [[ $# -gt 0 ]]
    do
        Option="$1"
        shift
        case $Option in
            -h|--help)
                PrintHelp
                exit 0
                ;;
            *)
                # implements the command if all options processed 
                COMMAND=$Option
                echo "---- Command $COMMAND with BSP \"$BspPath\""
                if [ "$COMMAND" == "help" ]
                then
                    PrintHelp
                elif [ "$COMMAND" == "setup" ]
                then
                   SetupCmd $@
                elif [ "$COMMAND" == "build" ]
                then
                   BuildCmd $@
                else
                    echo "---- ERROR: Unsupported command \"$COMMAND\""
                    PrintHelp
                    exit 1
                fi

                echo "---- Completed $SCRIPT_NAME $COMMAND"
                exit 0
                ;;


        esac
    done
fi