################################################################################
# (c) Copyright 2018 Xilinx, Inc. All rights reserved.
#
# This file contains confidential and proprietary information
# of Xilinx, Inc. and is protected under U.S. and
# international copyright and other intellectual property
# laws.
#
# DISCLAIMER
# This disclaimer is not a license and does not grant any
# rights to the materials distributed herewith. Except as
# otherwise provided in a valid license issued to you by
# Xilinx, and to the maximum extent permitted by applicable
# law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
# WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
# AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
# BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
# INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
# (2) Xilinx shall not be liable (whether in contract or tort,
# including negligence, or under any other theory of
# liability) for any loss or damage of any kind or nature
# related to, arising under or in connection with these
# materials, including for any direct, or any indirect,
# special, incidental, or consequential loss or damage
# (including loss of data, profits, goodwill, or any type of
# loss or damage suffered as a result of any action brought
# by a third party) even if such damage or loss was
# reasonably foreseeable or Xilinx had been advised of the
# possibility of the same.
#
# CRITICAL APPLICATIONS
# Xilinx products are not designed or intended to be fail-
# safe, or for use in any application requiring fail-safe
# performance, such as life-support or safety devices or
# systems, Class III medical devices, nuclear facilities,
# applications related to the deployment of airbags, or any
# other applications that could lead to death, personal
# injury, or severe property or environmental damage
# (individually and collectively, "Critical
# Applications"). Customer assumes the sole risk and
# liability of any use of Xilinx products in Critical
# Applications, subject only to applicable laws and
# regulations governing limitations on product liability.
#
# THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
# PART OF THIS FILE AT ALL TIMES.
################################################################################
# perl script that wraps petalinux build/creation for SD FEC driver project
################################################################################

# Imports
#use strict;
use Getopt::Long;
use File::Spec::Functions qw(rel2abs);
use File::Basename;
use File::Path qw(mkpath);
use Cwd 'abs_path';
use FindBin;
use IO::Handle;

# get_script_name
sub get_scriptname {

   # Script location and name
    my $scriptpath = rel2abs($0);
    my ($script, $scriptdir, $scriptext) = fileparse($scriptpath, qr/\.[^.]*/);
    my $scriptname = ($scriptext) ? "${script}${scriptext}" : $script;

    return $scriptname;
}

# do_system
sub do_system(@_) {
    my $cmd = shift;
    $result = system($cmd);


    if ($result != 0) {
        if ($? == 256 ) {
            # ignore for time being... this seems to be OK
            # printf "??? child exited with value %d\n", $? >> 8;
        } elsif ($? == -1) {
            die("failed to execute: $!\n");
        } elsif ($? & 127) {
            printf "child died with signal %d, %s coredump\n", ($? & 127),  ($? & 128) ? 'with' : 'without';
        } else {
            printf "child exited with value %d\n", $? >> 8;
        }
    }
}

# edit_rootfs_config
#
# add TCF and DROPBEAR options for debug
sub edit_rootfs_config {
    print("edit_rootfs_config\n");
    my $board    = shift;

    my $out_filename = "project-spec/configs/rootfs_config";
    my $in_filename  = "project-spec/configs/rootfs_config.old";
    do_system("cp -f $out_filename $in_filename");

    # open files
    open(my $src_fh,"<",$in_filename)  || die("*** cannot open $in_filename");
    open(my $dst_fh,">",$out_filename) || die("*** cannot open $out_filename");

    # do search and replace
    my $matches_found = 0;

    my @set_options = (
              "CONFIG_tcf-agent",
              "CONFIG_dropbear",
              "CONFIG_libsysfs",
              "CONFIG_libmetal",
          );
    my @clear_options = (
              "CONFIG_rfdc",
              "CONFIG_rfdc-intr",
              "CONFIG_rfdc-read-write",
              "CONFIG_rfdc-selftest",
              "CONFIG_peekpoke",
          );

    while(<$src_fh>) {

        foreach my $opt (@set_options) {
            $matches_found += s/# ${opt} is not set/${opt}=y/;
        }

        foreach my $opt (@clear_options) {
            $matches_found += s/${opt}=y/# ${opt} is not set/;
        }
        print $dst_fh $_;

    }

    # close files
    close($src_fh);
    close($dst_fh);

    # check matches_found
    printf("--- ${out_filename}: %d changes\n",$matches_found);

}

# create_app_bitbake
#
# follows instructions in Petalinux Tools Reference Guide (Steps to Adding Custom Applications)
sub create_app_bitbake {
    print("create_app_bitbake\n");
    my $app_name   = shift;
    my $depends    = shift;

    # open files
    my $recipe_dir   = "project-spec/meta-user/recipes-apps/${app_name}";
    my $out_filename = "${recipe_dir}/${app_name}.bb";
    my $files_dir    = "${recipe_dir}/files";

    open(my $dst_fh,">",$out_filename) || die("*** cannot open $out_filename");

    # template taken from default generated by Petalinux
    #
    #

    # part 1 : before source file list
    print $dst_fh <<EOF;
#
# This file is the $app_name recipe.
#

SUMMARY = "$app_name application"

LICENSE = "MIT"
LIC_FILES_CHKSUM = "file://\$\{COMMON_LICENSE_DIR\}/MIT;md5=0835ade698e0bcf8506ecda2f7b4f302"


EOF

    # output depends
    if ($depends ne "") {
        print $dst_fh "DEPENDS += \"${depends}\"\n\n";
    }

    # ouput all source files found in files (these must be copied there first)
    print $dst_fh "SRC_URI = \"\\\n";
    my $dh;
    opendir($dh,$files_dir) ||  die("cannot open directory: $files_dir");
    my @files = readdir($dh);
    closedir($dh);
    foreach $file (@files) {
        if (!(-d "$files_dir/$file")) {
            print $dst_fh "            file://${file} \\\n";
        }
    }
    print $dst_fh "          \"\n";

# part 2 : after source file list
print $dst_fh <<EOF;

S = "\$\{WORKDIR\}"

do_install() \{
         install -d \$\{D\}\$\{bindir\}
         install -m 0755 ${app_name} \$\{D\}\$\{bindir\}
\}

EOF

    # close file
    close($dst_fh);
}

# create_lib_bitbake
#
# follows instructions in Petalinux Tools Reference Guide (Steps to Adding Custom Applications)
sub create_lib_bitbake {
    print("create_app_bitbake\n");
    my $recipe_name  = shift;
    my $depends      = shift;

    # open files
    my $recipe_dir   = "project-spec/meta-user/recipes-libs/lib${recipe_name}";
    my $out_filename = "${recipe_dir}/lib${recipe_name}.bb";
    my $files_dir    = "${recipe_dir}/files";

    open(my $dst_fh,">",$out_filename) || die("*** cannot open $out_filename");

    # template taken from default generated by Petalinux
    #
    #

    # part 1 : before source file list
    print $dst_fh <<EOF;
#
# This file is the lib$recipe_name recipe.
#

SUMMARY = "A software library"
SECTION = "libs"
LICENSE = "MIT"
LIC_FILES_CHKSUM = "file://\$\{COMMON_LICENSE_DIR\}/MIT;md5=0835ade698e0bcf8506ecda2f7b4f302"

EOF

    # output depends
    if ($depends ne "") {
        print $dst_fh "DEPENDS += \"${depends}\"\n\n";
    }

    # ouput all source files found in files (these must be copied there first)
    print $dst_fh "SRC_URI = \"\\\n";
    my $dh;
    opendir($dh,$files_dir) ||  die("cannot open directory: $files_dir");
    my @files = readdir($dh);
    closedir($dh);
    foreach $file (@files) {
        if (!(-d "$files_dir/$file")) {
            print $dst_fh "            file://${file} \\\n";
        }
    }
    print $dst_fh "          \"\n";

# part 2 : after source file list
print $dst_fh <<EOF;

S = "\$\{WORKDIR\}"

PACKAGE_ARCH    = "\$\{MACHINE_ARCH\}"

PROVIDES        = "lib${recipe_name}"

do_install() \{
         install -d \$\{D\}\$\{libdir\}
         install -d \$\{D\}\$\{includedir\}
         oe_libinstall -so lib${recipe_name} \$\{D\}\$\{libdir\}
         install -d -m 0655 \$\{D\}\$\{includedir\}
         install -m 0644 \$\{S\}/*.h \$\{D\}\$\{includedir\}
\}

FILES_\$\{PN\} = "\$\{libdir\}/*.so.* \$\{includedir\}"
FILES_\$\{PN\}-dev = "\$\{libdir\}/*.so"

EOF

    # close file
    close($dst_fh);
}

# create_app_makefile
#
# follows template used when adding applications using petalinux-create -t apps
sub create_app_makefile {
    print("create_app_makefile\n");
    my $app_name   = shift;
    my $cflags     = shift;
    my $ldlibs     = shift;

    my $recipe_dir   = "project-spec/meta-user/recipes-apps/${app_name}";
    my $files_dir    = "${recipe_dir}/files";
    my $out_filename = "${files_dir}/Makefile";
    open(my $dst_fh,">",$out_filename) || die("*** cannot open $out_filename");

    # template taken from default generated by Petalinux
    #
    #

    print $dst_fh <<EOF;

APP = ${app_name}

# Add any other object files to this list below
EOF

    print $dst_fh "APP_OBJS = \n";

    opendir($dh,$files_dir) ||  die("cannot open directory: $files_dir");
    my @files = readdir($dh);
    closedir($dh);
    foreach $file (@files) {
        if (!(-d "$files_dir/$file")) {

            my($filename, $file_dir, $file_suffix) = fileparse("$files_dir\/$file", ".c");

            # if $file_suffix is an empty string something wrong
            if ($file_suffix eq ".c") {
                    print $dst_fh "APP_OBJS += ${filename}.o\n";
            }
        }
    }

    print $dst_fh <<EOF;

CFLAGS += ${cflags}
LDLIBS = ${ldlibs}

all: build

build: \$(APP)

\$(APP): \$(APP_OBJS)
	\$(CC) \$(LDFLAGS) -o \$@ \$(APP_OBJS) \$(LDLIBS)

clean:
	-rm -f \$(APP) *.elf *.gdb *.o
EOF

    # close file
    close($dst_fh);

}

# create_lib_makefile
#
# follows template used when adding libraries using petalinux-create -t apps
sub create_lib_makefile {
    print("create_lib_makefile\n");
    my $lib_name         = shift;
    my $cflags           = shift;
    my $ldlibs           = shift;

    my $recipe_dir   = "project-spec/meta-user/recipes-libs/lib${lib_name}";
    my $files_dir    = "${recipe_dir}/files";
    my $out_filename = "${files_dir}/Makefile";
    open(my $dst_fh,">",$out_filename) || die("*** cannot open $out_filename");

    # template taken from default generated by Petalinux
    #
    #

    print $dst_fh <<EOF;

APP = ${lib_name}

# Add any other object files to this list below
EOF

    print $dst_fh "LIB_OBJS = \n";

    opendir($dh,$files_dir) ||  die("cannot open directory: $files_dir");
    my @files = readdir($dh);
    closedir($dh);
    foreach $file (@files) {
        if (!(-d "$files_dir/$file")) {

            my($filename, $file_dir, $file_suffix) = fileparse("$files_dir\/$file", ".c");

            # if $file_suffix is an empty string something wrong
            if ($file_suffix eq ".c") {
                    print $dst_fh "LIB_OBJS += ${filename}.o\n";
            }
        }
    }

    print $dst_fh <<EOF;

LIBSOURCES=*.c
OUTS = *.o
NAME := ${lib_name}
MAJOR = 1.0
MINOR = 1
VERSION = \$(MAJOR).\$(MINOR)

LDLIBS = ${ldlibs}

all: lib\$(NAME).so

lib\$(NAME).so.\$(VERSION): \$(OUTS)
	\$(CC) \$(LDFLAGS) \$(OUTS) -shared -Wl,-soname,lib\$(NAME).so.\$(MAJOR) -o lib\$(NAME).so.\$(VERSION)

lib\$(NAME).so: lib\$(NAME).so.\$(VERSION)
	rm -f lib\$(NAME).so.\$(MAJOR) lib\$(NAME).so
	ln -s lib\$(NAME).so.\$(VERSION) lib\$(NAME).so.\$(MAJOR)
	ln -s lib\$(NAME).so.\$(MAJOR) lib\$(NAME).so

%.o: %.c
	\$(CC) \$(CFLAGS) -c -fPIC \$(LIBSOURCES)

clean:
	rm -rf *.o *.so *.so.*

EOF

    # close file
    close($dst_fh);
}

# get_tmp_dir
#
#
sub get_tmp_dir {
    print("get_tmp_dir\n");
    my $cfg_filename = "project-spec/configs/config";
    open(my $cfg_fh,"<",$cfg_filename) || die("*** cannot open $cfg_filename");

    my $tmp_dir;
    while (<$cfg_fh>) {
    	if (/CONFIG_TMP_DIR_LOCATION=\"(.*)\"/) {
    	    $tmp_dir = $1;
    	    print("CONFIG_TMP_DIR_LOCATION = $tmp_dir\n");
    	    close($cfg_fh);
    	    return $tmp_dir;
    	}
    }

    print("*** cannot find CONFIG_TMP_DIR_LOCATION\n");
    close($cfg_fh);
    return "";
}


# get_bsp_details
#
#
sub get_bsp_details {

    my $bsp_name = shift;

    my @bsp_tokens = split('-', $bsp_name);
    my $num_tokens = @bsp_tokens;


    my $board_name = "";
    my $manufacturer = $bsp_tokens[0];

    # the board names sometimes vary to include part numbers, es<#num>
    # this should help parse that
    for my $i (1 .. ($num_tokens - 3)) {
	if ($i == 1) {
	    $board_name = $bsp_tokens[$i];
	} else {
	    $board_name = $board_name . "-" . $bsp_tokens[$i];
	}
    }

    my $tool_version = $bsp_tokens[$num_tokens - 2];

    # rmove the lead "v" that must be present
    $tool_version =~ s/^v{1}//s;

    return ($manufacturer, $board_name, $tool_version) ;

}

# validate_generate_input_file
#
#
sub validate_generate_input_file {

    my $full_filename = shift;
    my $required_suffix = shift;

    if (!(-f "$full_filename")) {
        die("**** file does not exist (file: $full_filename)");
    }

    #${TODO} add check for hwdef when updating with zc1254 item
    my($filename, $file_dir, $file_suffix) = fileparse($full_filename, $required_suffix);

    # if $file_suffix is an empty string something wrong
    if ($file_suffix ne $required_suffix) {
        die("**** Invalid File Type (Required Suffix: $required_suffix) (Actual File: $filename)");
    }

}

sub setup_sync_sdfecusrintf_lib {

    $recipe_name = "sdfecusrintf";
    $lib_src_dir = "../sw_plnx/sdfec_usr_intf";
    $lib_dst_dir = "project-spec/meta-user/recipes-libs/lib${recipe_name}/files";

    my $cflags  = "-std=gnu99";
    my $ldlibs  = "";
    my $depends = "";

    do_system("mkdir -p ${lib_dst_dir}");

    # copy to petalinux
    do_system("cp -f ${lib_src_dir}/* ${lib_dst_dir}");

    create_lib_makefile($recipe_name,$cflags,$ldlibs);
    create_lib_bitbake($recipe_name,$depends);
}


sub setup_sync_sdfec_demo_app {

    $recipe_name = "sdfec-demo";
    $app_src_dir = "../sw_plnx/sdfec_demo";;
    $app_dst_dir = "project-spec/meta-user/recipes-apps/${recipe_name}/files";

    my $cflags  = "-std=gnu99";
    my $ldlibs  = "-lm -lmetal -lsdfecusrintf";
    my $depends = "libmetal libsdfecusrintf";


    do_system("petalinux-create -t apps -n ${recipe_name} --enable --force");
    do_system("cp -f ${app_src_dir}/* ${app_dst_dir}");
    do_system("rm -f ${app_dst_dir}/${recipe_name}.c");

    create_app_makefile($recipe_name,$cflags,$ldlibs);
    create_app_bitbake($recipe_name,$depends);
}


# plnx_generate
#
#
sub plnx_generate {
    my $usage = "
Generate Subcommand

Usage:
    $scriptname generate [options]

Where [options] are one of:

    -help    : output this help message
    -bsp     : full path for plnx bsp use to create project
    -stage   : define build stage to run
";

# Get input options
GetOptions('help|h'    => \$help,
           'bsp=s'     => \$bsp,
           'stage=i'   => \$stage,
       ) or die $usage;

    die $usage if $help;

    # if no input for stage do everything
    if (!($stage)) {
        $stage=0
    }

    # First the BSP
    $bsp = (abs_path($bsp));
    my $required_suffix = ".bsp";
    validate_generate_input_file($bsp, $required_suffix);

    if (($stage==0) || ($stage==1)) {

        print("--- GENERATE STAGE 1: Create target $plnx_proj_dir project from reference bsp\n");
        do_system("petalinux-create -t project -s $bsp -n $plnx_proj_dir --force");

    }

    # all the following stages will be done in the project directory
    chdir($plnx_proj_dir);

    if (($stage==0) || ($stage==2)) {
        print("--- GENERATE STAGE 2: Add Applications to build & Sync sources\n");
        setup_sync_sdfecusrintf_lib();
        setup_sync_sdfec_demo_app();
    }


    if (($stage==0) || ($stage==3)) {
        print("--- GENERATE STAGE 3: Edit rootfs configuration\n");
        edit_rootfs_config($board);
    };
    # out of project directory
    chdir("..");
}

# plnx_build
#
#
sub plnx_build {
    my $usage = "
Generate Subcommand

Usage:
    $scriptname build [options]

Where [options] are one of:

    -help        : output this help message
    -component|c : component to build
    -task|x      : build task
";

# Get input options
GetOptions('help|h'        => \$help,
           'component|c=s' => \$component,
           'task|x=s'      => \$task,
           'target|w=s'    => \$target_dir,
       ) or die $usage;

    die $usage if $help;

    $plnx_proj_dir = (abs_path($plnx_proj_dir));
    chdir($plnx_proj_dir);

    print("component:$component, work_dir:$work_dir, stage: $stage\n");
    my $cmd = "petalinux-build";

    if ($component ne "") {
        $cmd = "$cmd -c $component";
    }

    if ($task ne "") {
        $cmd = "$cmd -x $task";
    }

    print("Execute Command: $cmd");

    do_system($cmd);

    do_system("cp -f ./pre-built/linux/images/zynqmp_fsbl.elf ./images/linux/");
}


#    plnx_package
#
#
sub plnx_package {
    my $usage = "
Package Subcommand

Creates an SD CARD
Usage:
    $scriptname package [options]

Where [options] are one of:

    -help    : output this help message
    -bsp     : path for plnx bsp use to create project
    -bit     : full path fo the bitstream to use in the BOOT.BIN
";

# Get input options
GetOptions('help|h'    => \$help,
           'bsp=s'     => \$bsp,
           'bit=s'     => \$bitstream,
       ) or die $usage;

    die $usage if $help;

    print("bsp: $bsp bit: $bitstream");
    $bsp = (abs_path($bsp));
    my $required_suffix = ".bsp";
    # get and validate the bsp details
    my $bsp_name = fileparse($bsp, $required_suffix);
    validate_bsp_name($bsp_name);

    print("bitstream: $bitstream");
    if (!(-f "$bitstream")) {
        die("**** Bitstream does not exist (bit: $bitstream)");
    }

    my($manufacturer, $board, $tool_version) = get_bsp_details($bsp_name);
    my $plnx_proj_dir = "plnx_sdfec-$board-$tool_version";

    $plnx_proj_dir = (abs_path($plnx_proj_dir));
    chdir($plnx_proj_dir);

    my $cmd = "petalinux-package --boot --format BIN --fsbl images/linux/zynqmp_fsbl.elf --u-boot";
    $cmd = "$cmd --fpga $bitstream";
    $cmd = "$cmd --force";
    do_system($cmd);
}

################################################################################
# Main Script
#
#

$scriptname = get_scriptname();

$plnx_proj_dir = "plnx_sdfec_demo";

# Subcommand Usage
my $sub_cmd_usage = "
SD FEC Petalinux Build Scripts

Usage:
    $scriptname <subcmd> [options]

Where <subcmd> is one of:

    help     : output this help message
    generate : generate SD-FEC petalinux project
    build    : build the SD-FEC petalinux project
    package  : package SD-FEC petalinux project into a petalinux BSP

for additional help on subcomamnds use:

    $scriptname <subcmd> -help

";


# Subcomamnd is first agument
my $sub_cmd = $ARGV[0];

do_system("petalinux-util --webtalk off");

# Execute Subcommand
if ($sub_cmd eq 'help') {
    die $sub_cmd_usage;
} elsif ($sub_cmd eq 'generate') {
    plnx_generate();
} elsif ($sub_cmd eq 'build') {
    plnx_build();
} elsif ($sub_cmd eq 'package') {
    plnx_package();
} else {
    print("*** Unexpected Sub-command: $sub_cmd\n");
    die $sub_cmd_usage;
}
